package coffee._finally.keycloak_metrics;

import io.prometheus.client.Collector;
import io.prometheus.client.CollectorRegistry;
import io.prometheus.client.Counter;
import io.prometheus.client.Gauge;
import io.prometheus.client.exporter.common.TextFormat;

import java.io.*;
import java.util.HashMap;
import java.util.Map;

public class Metrics {

    private static Metrics METRICS_INSTANCE = null;
    private static Map<String, Collector> metrics = new HashMap();
    private static CollectorRegistry registry = new CollectorRegistry();

    private Metrics() {
        initializeMetrics();
    }

    private void initializeMetrics() {
        metrics.put("realm", Gauge.build()
                .namespace("keycloak")
                .name("realm_count")
                .labelNames("realm")
                .help("Number of realms in keycloak")
                .register(registry));
        metrics.put("realm_role", Gauge.build()
                .namespace("keycloak")
                .name("role_count")
                .labelNames("realm")
                .help("Number of roles in a realm")
                .register(registry));
        metrics.put("admin_event", Counter.build()
                .namespace("keycloak").name("admin_event_count")
                .labelNames("realm", "operation", "resource")
                .help("Number and type of admin events since startup")
                .register(registry));
        metrics.put("user_event", Counter.build()
                .namespace("keycloak").name("user_event_count")
                .labelNames("realm", "type")
                .help("Number and type of user events since startup")
                .register(registry));
    }

    public static synchronized Metrics getInstance() {
        if (METRICS_INSTANCE == null) {
            METRICS_INSTANCE = new Metrics();
        }
        return METRICS_INSTANCE;
    }

    public CollectorRegistry getRegistry() {
        return registry;
    }

    public static void increment(String metric) {
        Collector collector = metrics.get(metric);
        if (collector == null) {
            return;
        }
        if (collector instanceof Gauge) {
            ((Gauge) collector).inc();
            return;
        }
        if (collector instanceof Counter) {
            ((Counter) collector).inc();
        }
    }

    public static void incrementLabelled(String metric, String... labels) {
        Collector collector = metrics.get(metric);
        if (collector == null) {
            return;
        }
        if (collector instanceof Gauge) {
            ((Gauge) collector).labels(labels).inc();
            return;
        }
        if (collector instanceof Counter) {
            ((Counter) collector).labels(labels).inc();
        }
    }

    public static void decrement(String metric) {
        Collector collector = metrics.get(metric);
        if (collector == null) {
            return;
        }
        if (collector instanceof Gauge) {
            ((Gauge) collector).dec();
        }
    }

    public static void decrementLabelled(String metric, String... labels) {
        Collector collector = metrics.get(metric);
        if (collector == null) {
            return;
        }
        if (collector instanceof Gauge) {
            ((Gauge) collector).labels(labels).dec();
        }
    }

    public void export(final OutputStream stream) throws IOException {
        final Writer writer = new BufferedWriter(new OutputStreamWriter(stream));
        TextFormat.writeOpenMetrics100(writer, registry.metricFamilySamples());
        writer.flush();
    }
}
