package coffee._finally.keycloak_metrics;

import io.prometheus.client.exporter.HTTPServer;
import org.keycloak.Config;
import org.keycloak.models.KeycloakSession;
import org.keycloak.models.KeycloakSessionFactory;
import org.keycloak.provider.Provider;
import org.keycloak.provider.ProviderFactory;

import java.io.IOException;

public class MetricsServerProviderFactory implements ServerProviderFactory {

    private static HTTPServer metricsServer = null;

    @Override
    public Provider create(KeycloakSession session) {
        return new MetricsServerProvider();
    }

    /**
     * Only called once when the factory is first created.  This config is pulled from keycloak_server.json
     *
     * @param config
     */
    @Override
    public void init(Config.Scope config) {
        // intentionally left blank
    }

    /**
     * Called after all provider factories have been initialized
     *
     * @param factory
     */
    @Override
    public void postInit(KeycloakSessionFactory factory) {
        if (metricsServer == null) {
            try {
                metricsServer = new HTTPServer.Builder()
                        .withPort(9400)
                        .withRegistry(Metrics.getInstance().getRegistry())
                        .build();
            } catch (IOException io) {
                System.err.println("Unable to start HTTP server for metrics on port 9400: " + io.toString());
            }
        }
    }

    /**
     * This is called when the server shuts down.
     */
    @Override
    public void close() {
        if (metricsServer != null) {
            metricsServer.close();
        }
    }

    @Override
    public String getId() {
        return "metrics-server-provider-spi";
    }
}
